<?php
/*
 * Copyright 2021 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/googleapis/googleapis/blob/master/google/pubsub/v1/schema.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Cloud\PubSub\V1\Gapic;

use Google\ApiCore\ApiException;
use Google\ApiCore\Call;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\PathTemplate;
use Google\ApiCore\RequestParamsHeaderDescriptor;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;
use Google\Cloud\Iam\V1\GetIamPolicyRequest;
use Google\Cloud\Iam\V1\GetPolicyOptions;
use Google\Cloud\Iam\V1\Policy;
use Google\Cloud\Iam\V1\SetIamPolicyRequest;
use Google\Cloud\Iam\V1\TestIamPermissionsRequest;
use Google\Cloud\Iam\V1\TestIamPermissionsResponse;
use Google\Cloud\PubSub\V1\CommitSchemaRequest;
use Google\Cloud\PubSub\V1\CreateSchemaRequest;
use Google\Cloud\PubSub\V1\DeleteSchemaRequest;
use Google\Cloud\PubSub\V1\DeleteSchemaRevisionRequest;
use Google\Cloud\PubSub\V1\Encoding;
use Google\Cloud\PubSub\V1\GetSchemaRequest;
use Google\Cloud\PubSub\V1\ListSchemaRevisionsRequest;
use Google\Cloud\PubSub\V1\ListSchemaRevisionsResponse;
use Google\Cloud\PubSub\V1\ListSchemasRequest;
use Google\Cloud\PubSub\V1\ListSchemasResponse;
use Google\Cloud\PubSub\V1\RollbackSchemaRequest;
use Google\Cloud\PubSub\V1\Schema;
use Google\Cloud\PubSub\V1\ValidateMessageRequest;
use Google\Cloud\PubSub\V1\ValidateMessageResponse;
use Google\Cloud\PubSub\V1\ValidateSchemaRequest;
use Google\Cloud\PubSub\V1\ValidateSchemaResponse;
use Google\Protobuf\FieldMask;
use Google\Protobuf\GPBEmpty;

/**
 * Service Description: Service for doing schema-related operations.
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods. Sample code to get started:
 *
 * ```
 * $schemaServiceClient = new SchemaServiceClient();
 * try {
 *     $formattedName = $schemaServiceClient->schemaName('[PROJECT]', '[SCHEMA]');
 *     $schema = new Schema();
 *     $response = $schemaServiceClient->commitSchema($formattedName, $schema);
 * } finally {
 *     $schemaServiceClient->close();
 * }
 * ```
 *
 * Many parameters require resource names to be formatted in a particular way. To
 * assist with these names, this class includes a format method for each type of
 * name, and additionally a parseName method to extract the individual identifiers
 * contained within formatted names that are returned by the API.
 *
 * @deprecated Please use the new service client {@see \Google\Cloud\PubSub\V1\Client\SchemaServiceClient}.
 */
class SchemaServiceGapicClient
{
    use GapicClientTrait;

    /** The name of the service. */
    const SERVICE_NAME = 'google.pubsub.v1.SchemaService';

    /**
     * The default address of the service.
     *
     * @deprecated SERVICE_ADDRESS_TEMPLATE should be used instead.
     */
    const SERVICE_ADDRESS = 'pubsub.googleapis.com';

    /** The address template of the service. */
    private const SERVICE_ADDRESS_TEMPLATE = 'pubsub.UNIVERSE_DOMAIN';

    /** The default port of the service. */
    const DEFAULT_SERVICE_PORT = 443;

    /** The name of the code generator, to be included in the agent header. */
    const CODEGEN_NAME = 'gapic';

    /** The default scopes required by the service. */
    public static $serviceScopes = [
        'https://www.googleapis.com/auth/cloud-platform',
        'https://www.googleapis.com/auth/pubsub',
    ];

    private static $projectNameTemplate;

    private static $schemaNameTemplate;

    private static $pathTemplateMap;

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'apiEndpoint' => self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' => __DIR__ . '/../resources/schema_service_client_config.json',
            'descriptorsConfigPath' => __DIR__ . '/../resources/schema_service_descriptor_config.php',
            'gcpApiConfigPath' => __DIR__ . '/../resources/schema_service_grpc_config.json',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' => __DIR__ . '/../resources/schema_service_rest_client_config.php',
                ],
            ],
        ];
    }

    private static function getProjectNameTemplate()
    {
        if (self::$projectNameTemplate == null) {
            self::$projectNameTemplate = new PathTemplate('projects/{project}');
        }

        return self::$projectNameTemplate;
    }

    private static function getSchemaNameTemplate()
    {
        if (self::$schemaNameTemplate == null) {
            self::$schemaNameTemplate = new PathTemplate('projects/{project}/schemas/{schema}');
        }

        return self::$schemaNameTemplate;
    }

    private static function getPathTemplateMap()
    {
        if (self::$pathTemplateMap == null) {
            self::$pathTemplateMap = [
                'project' => self::getProjectNameTemplate(),
                'schema' => self::getSchemaNameTemplate(),
            ];
        }

        return self::$pathTemplateMap;
    }

    /**
     * Formats a string containing the fully-qualified path to represent a project
     * resource.
     *
     * @param string $project
     *
     * @return string The formatted project resource.
     */
    public static function projectName($project)
    {
        return self::getProjectNameTemplate()->render([
            'project' => $project,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a schema
     * resource.
     *
     * @param string $project
     * @param string $schema
     *
     * @return string The formatted schema resource.
     */
    public static function schemaName($project, $schema)
    {
        return self::getSchemaNameTemplate()->render([
            'project' => $project,
            'schema' => $schema,
        ]);
    }

    /**
     * Parses a formatted name string and returns an associative array of the components in the name.
     * The following name formats are supported:
     * Template: Pattern
     * - project: projects/{project}
     * - schema: projects/{project}/schemas/{schema}
     *
     * The optional $template argument can be supplied to specify a particular pattern,
     * and must match one of the templates listed above. If no $template argument is
     * provided, or if the $template argument does not match one of the templates
     * listed, then parseName will check each of the supported templates, and return
     * the first match.
     *
     * @param string $formattedName The formatted name string
     * @param string $template      Optional name of template to match
     *
     * @return array An associative array from name component IDs to component values.
     *
     * @throws ValidationException If $formattedName could not be matched.
     */
    public static function parseName($formattedName, $template = null)
    {
        $templateMap = self::getPathTemplateMap();
        if ($template) {
            if (!isset($templateMap[$template])) {
                throw new ValidationException("Template name $template does not exist");
            }

            return $templateMap[$template]->match($formattedName);
        }

        foreach ($templateMap as $templateName => $pathTemplate) {
            try {
                return $pathTemplate->match($formattedName);
            } catch (ValidationException $ex) {
                // Swallow the exception to continue trying other path templates
            }
        }

        throw new ValidationException("Input did not match any known format. Input: $formattedName");
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $apiEndpoint
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'pubsub.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. May be either the string
     *           `rest` or `grpc`. Defaults to `grpc` if gRPC support is detected on the system.
     *           *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $apiEndpoint setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'grpc' => [...],
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\GrpcTransport::build()} and
     *           {@see \Google\ApiCore\Transport\RestTransport::build()} methods for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
    }

    /**
     * Commits a new schema revision to an existing schema.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedName = $schemaServiceClient->schemaName('[PROJECT]', '[SCHEMA]');
     *     $schema = new Schema();
     *     $response = $schemaServiceClient->commitSchema($formattedName, $schema);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $name         Required. The name of the schema we are revising.
     *                             Format is `projects/{project}/schemas/{schema}`.
     * @param Schema $schema       Required. The schema revision to commit.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\PubSub\V1\Schema
     *
     * @throws ApiException if the remote call fails
     */
    public function commitSchema($name, $schema, array $optionalArgs = [])
    {
        $request = new CommitSchemaRequest();
        $requestParamHeaders = [];
        $request->setName($name);
        $request->setSchema($schema);
        $requestParamHeaders['name'] = $name;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('CommitSchema', Schema::class, $optionalArgs, $request)->wait();
    }

    /**
     * Creates a schema.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedParent = $schemaServiceClient->projectName('[PROJECT]');
     *     $schema = new Schema();
     *     $response = $schemaServiceClient->createSchema($formattedParent, $schema);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $parent       Required. The name of the project in which to create the schema.
     *                             Format is `projects/{project-id}`.
     * @param Schema $schema       Required. The schema object to create.
     *
     *                             This schema's `name` parameter is ignored. The schema object returned
     *                             by CreateSchema will have a `name` made using the given `parent` and
     *                             `schema_id`.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type string $schemaId
     *           The ID to use for the schema, which will become the final component of
     *           the schema's resource name.
     *
     *           See https://cloud.google.com/pubsub/docs/admin#resource_names for resource
     *           name constraints.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\PubSub\V1\Schema
     *
     * @throws ApiException if the remote call fails
     */
    public function createSchema($parent, $schema, array $optionalArgs = [])
    {
        $request = new CreateSchemaRequest();
        $requestParamHeaders = [];
        $request->setParent($parent);
        $request->setSchema($schema);
        $requestParamHeaders['parent'] = $parent;
        if (isset($optionalArgs['schemaId'])) {
            $request->setSchemaId($optionalArgs['schemaId']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('CreateSchema', Schema::class, $optionalArgs, $request)->wait();
    }

    /**
     * Deletes a schema.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedName = $schemaServiceClient->schemaName('[PROJECT]', '[SCHEMA]');
     *     $schemaServiceClient->deleteSchema($formattedName);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $name         Required. Name of the schema to delete.
     *                             Format is `projects/{project}/schemas/{schema}`.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @throws ApiException if the remote call fails
     */
    public function deleteSchema($name, array $optionalArgs = [])
    {
        $request = new DeleteSchemaRequest();
        $requestParamHeaders = [];
        $request->setName($name);
        $requestParamHeaders['name'] = $name;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('DeleteSchema', GPBEmpty::class, $optionalArgs, $request)->wait();
    }

    /**
     * Deletes a specific schema revision.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedName = $schemaServiceClient->schemaName('[PROJECT]', '[SCHEMA]');
     *     $revisionId = 'revision_id';
     *     $response = $schemaServiceClient->deleteSchemaRevision($formattedName, $revisionId);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $name         Required. The name of the schema revision to be deleted, with a revision ID
     *                             explicitly included.
     *
     *                             Example: `projects/123/schemas/my-schema&#64;c7cfa2a8`
     * @param string $revisionId   Optional. This field is deprecated and should not be used for specifying
     *                             the revision ID. The revision ID should be specified via the `name`
     *                             parameter.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\PubSub\V1\Schema
     *
     * @throws ApiException if the remote call fails
     */
    public function deleteSchemaRevision($name, $revisionId, array $optionalArgs = [])
    {
        $request = new DeleteSchemaRevisionRequest();
        $requestParamHeaders = [];
        $request->setName($name);
        $request->setRevisionId($revisionId);
        $requestParamHeaders['name'] = $name;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('DeleteSchemaRevision', Schema::class, $optionalArgs, $request)->wait();
    }

    /**
     * Gets a schema.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedName = $schemaServiceClient->schemaName('[PROJECT]', '[SCHEMA]');
     *     $response = $schemaServiceClient->getSchema($formattedName);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $name         Required. The name of the schema to get.
     *                             Format is `projects/{project}/schemas/{schema}`.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type int $view
     *           The set of fields to return in the response. If not set, returns a Schema
     *           with all fields filled out. Set to `BASIC` to omit the `definition`.
     *           For allowed values, use constants defined on {@see \Google\Cloud\PubSub\V1\SchemaView}
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\PubSub\V1\Schema
     *
     * @throws ApiException if the remote call fails
     */
    public function getSchema($name, array $optionalArgs = [])
    {
        $request = new GetSchemaRequest();
        $requestParamHeaders = [];
        $request->setName($name);
        $requestParamHeaders['name'] = $name;
        if (isset($optionalArgs['view'])) {
            $request->setView($optionalArgs['view']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('GetSchema', Schema::class, $optionalArgs, $request)->wait();
    }

    /**
     * Lists all schema revisions for the named schema.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedName = $schemaServiceClient->schemaName('[PROJECT]', '[SCHEMA]');
     *     // Iterate over pages of elements
     *     $pagedResponse = $schemaServiceClient->listSchemaRevisions($formattedName);
     *     foreach ($pagedResponse->iteratePages() as $page) {
     *         foreach ($page as $element) {
     *             // doSomethingWith($element);
     *         }
     *     }
     *     // Alternatively:
     *     // Iterate through all elements
     *     $pagedResponse = $schemaServiceClient->listSchemaRevisions($formattedName);
     *     foreach ($pagedResponse->iterateAllElements() as $element) {
     *         // doSomethingWith($element);
     *     }
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $name         Required. The name of the schema to list revisions for.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type int $view
     *           The set of Schema fields to return in the response. If not set, returns
     *           Schemas with `name` and `type`, but not `definition`. Set to `FULL` to
     *           retrieve all fields.
     *           For allowed values, use constants defined on {@see \Google\Cloud\PubSub\V1\SchemaView}
     *     @type int $pageSize
     *           The maximum number of resources contained in the underlying API
     *           response. The API may return fewer values in a page, even if
     *           there are additional values to be retrieved.
     *     @type string $pageToken
     *           A page token is used to specify a page of values to be returned.
     *           If no page token is specified (the default), the first page
     *           of values will be returned. Any page token used here must have
     *           been generated by a previous call to the API.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\PagedListResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function listSchemaRevisions($name, array $optionalArgs = [])
    {
        $request = new ListSchemaRevisionsRequest();
        $requestParamHeaders = [];
        $request->setName($name);
        $requestParamHeaders['name'] = $name;
        if (isset($optionalArgs['view'])) {
            $request->setView($optionalArgs['view']);
        }

        if (isset($optionalArgs['pageSize'])) {
            $request->setPageSize($optionalArgs['pageSize']);
        }

        if (isset($optionalArgs['pageToken'])) {
            $request->setPageToken($optionalArgs['pageToken']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->getPagedListResponse('ListSchemaRevisions', $optionalArgs, ListSchemaRevisionsResponse::class, $request);
    }

    /**
     * Lists schemas in a project.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedParent = $schemaServiceClient->projectName('[PROJECT]');
     *     // Iterate over pages of elements
     *     $pagedResponse = $schemaServiceClient->listSchemas($formattedParent);
     *     foreach ($pagedResponse->iteratePages() as $page) {
     *         foreach ($page as $element) {
     *             // doSomethingWith($element);
     *         }
     *     }
     *     // Alternatively:
     *     // Iterate through all elements
     *     $pagedResponse = $schemaServiceClient->listSchemas($formattedParent);
     *     foreach ($pagedResponse->iterateAllElements() as $element) {
     *         // doSomethingWith($element);
     *     }
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $parent       Required. The name of the project in which to list schemas.
     *                             Format is `projects/{project-id}`.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type int $view
     *           The set of Schema fields to return in the response. If not set, returns
     *           Schemas with `name` and `type`, but not `definition`. Set to `FULL` to
     *           retrieve all fields.
     *           For allowed values, use constants defined on {@see \Google\Cloud\PubSub\V1\SchemaView}
     *     @type int $pageSize
     *           The maximum number of resources contained in the underlying API
     *           response. The API may return fewer values in a page, even if
     *           there are additional values to be retrieved.
     *     @type string $pageToken
     *           A page token is used to specify a page of values to be returned.
     *           If no page token is specified (the default), the first page
     *           of values will be returned. Any page token used here must have
     *           been generated by a previous call to the API.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\ApiCore\PagedListResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function listSchemas($parent, array $optionalArgs = [])
    {
        $request = new ListSchemasRequest();
        $requestParamHeaders = [];
        $request->setParent($parent);
        $requestParamHeaders['parent'] = $parent;
        if (isset($optionalArgs['view'])) {
            $request->setView($optionalArgs['view']);
        }

        if (isset($optionalArgs['pageSize'])) {
            $request->setPageSize($optionalArgs['pageSize']);
        }

        if (isset($optionalArgs['pageToken'])) {
            $request->setPageToken($optionalArgs['pageToken']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->getPagedListResponse('ListSchemas', $optionalArgs, ListSchemasResponse::class, $request);
    }

    /**
     * Creates a new schema revision that is a copy of the provided revision_id.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedName = $schemaServiceClient->schemaName('[PROJECT]', '[SCHEMA]');
     *     $revisionId = 'revision_id';
     *     $response = $schemaServiceClient->rollbackSchema($formattedName, $revisionId);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $name         Required. The schema being rolled back with revision id.
     * @param string $revisionId   Required. The revision ID to roll back to.
     *                             It must be a revision of the same schema.
     *
     *                             Example: c7cfa2a8
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\PubSub\V1\Schema
     *
     * @throws ApiException if the remote call fails
     */
    public function rollbackSchema($name, $revisionId, array $optionalArgs = [])
    {
        $request = new RollbackSchemaRequest();
        $requestParamHeaders = [];
        $request->setName($name);
        $request->setRevisionId($revisionId);
        $requestParamHeaders['name'] = $name;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('RollbackSchema', Schema::class, $optionalArgs, $request)->wait();
    }

    /**
     * Validates a message against a schema.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedParent = $schemaServiceClient->projectName('[PROJECT]');
     *     $response = $schemaServiceClient->validateMessage($formattedParent);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $parent       Required. The name of the project in which to validate schemas.
     *                             Format is `projects/{project-id}`.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type string $name
     *           Name of the schema against which to validate.
     *
     *           Format is `projects/{project}/schemas/{schema}`.
     *     @type Schema $schema
     *           Ad-hoc schema against which to validate
     *     @type string $message
     *           Message to validate against the provided `schema_spec`.
     *     @type int $encoding
     *           The encoding expected for messages
     *           For allowed values, use constants defined on {@see \Google\Cloud\PubSub\V1\Encoding}
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\PubSub\V1\ValidateMessageResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function validateMessage($parent, array $optionalArgs = [])
    {
        $request = new ValidateMessageRequest();
        $requestParamHeaders = [];
        $request->setParent($parent);
        $requestParamHeaders['parent'] = $parent;
        if (isset($optionalArgs['name'])) {
            $request->setName($optionalArgs['name']);
        }

        if (isset($optionalArgs['schema'])) {
            $request->setSchema($optionalArgs['schema']);
        }

        if (isset($optionalArgs['message'])) {
            $request->setMessage($optionalArgs['message']);
        }

        if (isset($optionalArgs['encoding'])) {
            $request->setEncoding($optionalArgs['encoding']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('ValidateMessage', ValidateMessageResponse::class, $optionalArgs, $request)->wait();
    }

    /**
     * Validates a schema.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $formattedParent = $schemaServiceClient->projectName('[PROJECT]');
     *     $schema = new Schema();
     *     $response = $schemaServiceClient->validateSchema($formattedParent, $schema);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $parent       Required. The name of the project in which to validate schemas.
     *                             Format is `projects/{project-id}`.
     * @param Schema $schema       Required. The schema object to validate.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\PubSub\V1\ValidateSchemaResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function validateSchema($parent, $schema, array $optionalArgs = [])
    {
        $request = new ValidateSchemaRequest();
        $requestParamHeaders = [];
        $request->setParent($parent);
        $request->setSchema($schema);
        $requestParamHeaders['parent'] = $parent;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('ValidateSchema', ValidateSchemaResponse::class, $optionalArgs, $request)->wait();
    }

    /**
     * Gets the access control policy for a resource. Returns an empty policy
    if the resource exists and does not have a policy set.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $resource = 'resource';
     *     $response = $schemaServiceClient->getIamPolicy($resource);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $resource     REQUIRED: The resource for which the policy is being requested.
     *                             See the operation documentation for the appropriate value for this field.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type GetPolicyOptions $options
     *           OPTIONAL: A `GetPolicyOptions` object for specifying options to
     *           `GetIamPolicy`.
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Iam\V1\Policy
     *
     * @throws ApiException if the remote call fails
     */
    public function getIamPolicy($resource, array $optionalArgs = [])
    {
        $request = new GetIamPolicyRequest();
        $requestParamHeaders = [];
        $request->setResource($resource);
        $requestParamHeaders['resource'] = $resource;
        if (isset($optionalArgs['options'])) {
            $request->setOptions($optionalArgs['options']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('GetIamPolicy', Policy::class, $optionalArgs, $request, Call::UNARY_CALL, 'google.iam.v1.IAMPolicy')->wait();
    }

    /**
     * Sets the access control policy on the specified resource. Replaces
    any existing policy.

    Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED`
    errors.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $resource = 'resource';
     *     $policy = new Policy();
     *     $response = $schemaServiceClient->setIamPolicy($resource, $policy);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string $resource     REQUIRED: The resource for which the policy is being specified.
     *                             See the operation documentation for the appropriate value for this field.
     * @param Policy $policy       REQUIRED: The complete policy to be applied to the `resource`. The size of
     *                             the policy is limited to a few 10s of KB. An empty policy is a
     *                             valid policy but certain Cloud Platform services (such as Projects)
     *                             might reject them.
     * @param array  $optionalArgs {
     *     Optional.
     *
     *     @type FieldMask $updateMask
     *           OPTIONAL: A FieldMask specifying which fields of the policy to modify. Only
     *           the fields in the mask will be modified. If no mask is provided, the
     *           following default mask is used:
     *
     *           `paths: "bindings, etag"`
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Iam\V1\Policy
     *
     * @throws ApiException if the remote call fails
     */
    public function setIamPolicy($resource, $policy, array $optionalArgs = [])
    {
        $request = new SetIamPolicyRequest();
        $requestParamHeaders = [];
        $request->setResource($resource);
        $request->setPolicy($policy);
        $requestParamHeaders['resource'] = $resource;
        if (isset($optionalArgs['updateMask'])) {
            $request->setUpdateMask($optionalArgs['updateMask']);
        }

        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('SetIamPolicy', Policy::class, $optionalArgs, $request, Call::UNARY_CALL, 'google.iam.v1.IAMPolicy')->wait();
    }

    /**
     * Returns permissions that a caller has on the specified resource. If the
    resource does not exist, this will return an empty set of
    permissions, not a `NOT_FOUND` error.

    Note: This operation is designed to be used for building
    permission-aware UIs and command-line tools, not for authorization
    checking. This operation may "fail open" without warning.
     *
     * Sample code:
     * ```
     * $schemaServiceClient = new SchemaServiceClient();
     * try {
     *     $resource = 'resource';
     *     $permissions = [];
     *     $response = $schemaServiceClient->testIamPermissions($resource, $permissions);
     * } finally {
     *     $schemaServiceClient->close();
     * }
     * ```
     *
     * @param string   $resource     REQUIRED: The resource for which the policy detail is being requested.
     *                               See the operation documentation for the appropriate value for this field.
     * @param string[] $permissions  The set of permissions to check for the `resource`. Permissions with
     *                               wildcards (such as '*' or 'storage.*') are not allowed. For more
     *                               information see
     *                               [IAM Overview](https://cloud.google.com/iam/docs/overview#permissions).
     * @param array    $optionalArgs {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return \Google\Cloud\Iam\V1\TestIamPermissionsResponse
     *
     * @throws ApiException if the remote call fails
     */
    public function testIamPermissions($resource, $permissions, array $optionalArgs = [])
    {
        $request = new TestIamPermissionsRequest();
        $requestParamHeaders = [];
        $request->setResource($resource);
        $request->setPermissions($permissions);
        $requestParamHeaders['resource'] = $resource;
        $requestParams = new RequestParamsHeaderDescriptor($requestParamHeaders);
        $optionalArgs['headers'] = isset($optionalArgs['headers']) ? array_merge($requestParams->getHeader(), $optionalArgs['headers']) : $requestParams->getHeader();
        return $this->startCall('TestIamPermissions', TestIamPermissionsResponse::class, $optionalArgs, $request, Call::UNARY_CALL, 'google.iam.v1.IAMPolicy')->wait();
    }
}
